/****************************************************************************
 **
 **  Copyright (C) 2007 Helge Kortendieck, qvtwfc@kasmithee.de
 **
 **  Main program 2, test mode, no save (pos only), just play the flash cards
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include "qvtwfct.h"

Qvtwfct::Qvtwfct(QWidget *parent)
        : QWidget(parent)
{
  ui.setupUi(this);

  settingsRead();

  connect(ui.pB_Next, SIGNAL( clicked() ), this, SLOT( fcLoopNext() ) );

  rwf = new rw();                                           // setup read file qobject
  connect( rwf, SIGNAL( sigReadAbort( const QString&, const QString&, int ) ), this, SLOT( readAbort( const QString&, const QString&, int ) ) ); // fatal error opening file
  connect( rwf, SIGNAL( sigReadWErr( const QString&, const QString&, int, const QString&, const QString& ) ), this, SLOT( readWErr( const QString&, const QString&, int, const QString&, const QString& ) ) );    // errors while reading file
  connect( rwf, SIGNAL( sigReadOk( const QString&, const QString& ) ), this, SLOT( readOk( const QString&, const QString& ) ) );  // finished reading file, no errors

  fcClear();

  iIterA = 0;                                       // num words to ask given as command line argument
  sFileA = "";                                      // vocabulary file to use given as command line argument
  QStringList slArgs = QApplication::arguments();   // get command line args
  iArgs = slArgs.count();
  for( int i = 1; i < iArgs; ++i ) {
    sD = slArgs.at( i );
    if( sD == "-a" ) {                              // found -a, command option for words to ask
      if( i + 1 < iArgs ) {                         // is another argument following the '-a'
        iD = slArgs.at( i + 1 ).toInt( &bOk, 10 );
        if( bOk ) {
          iIterA = slArgs.at( i + 1 ).toInt();
          i = i + 1;
        }
      }
    } else if( sD != "-t" ) {           // interpret as file provided by command line arguments
      QFileInfo qfi( slArgs.at( i ) );
      if( qfi.exists() and qfi.isReadable() ) {
        sFileA = sD;
      } else {
        QMessageBox::warning( this, tr( "Invalid file !" ), 
                                    tr( "The vocabulary file given as command argument: \n" ) + slArgs.at( i ) + 
                                        "\nis either not readable or does not exist at all. \n \n"
                                        "Argument ignored !" );
      }
    }
  }
  initDelay();                              // short timer to allow GUI to come up before proceeding...
}

void Qvtwfct::settingsRead()
{
  QSettings settings("qvtwfc", "qvtwfc");
  settings.beginGroup( "mainwindow" );
  sFile = settings.value( "file", "" ).toString();
  settings.endGroup();
  settings.beginGroup( "flash_card" );
  ptPos = settings.value( "pos", QPoint(0, 0) ).toPoint();
  seSize = settings.value( "size", QSize(800, 600) ).toSize();
  sFileT = settings.value( "filetest", "" ).toString();
  iIterT = settings.value( "iter", "2" ).toInt();
  settings.endGroup();

  resize( seSize);
  move(ptPos);
}

void Qvtwfct::settingsWrite()
{
  QSettings settings("qvtwfc", "qvtwfc");
  settings.beginGroup( "flash_card" );
  settings.setValue( "pos", pos() );
  settings.setValue( "size", size() );
  settings.endGroup();
  settings.sync();
  quitDo();
}

void Qvtwfct::quitPrep()
{
  if( ptPos != pos() or seSize != size() ) {    // only write settings if required
    settingsWrite();
  } else {
    quitDo();
  }
}

void Qvtwfct::quitDo()
{
  exit(0);
}

void Qvtwfct::initDelay()
{
  QTimer::singleShot(200, this, SLOT( init() ) );
}

void Qvtwfct::init()
{
  if( sFileA != "" ) {                      // 1. use voc file provided as command line arg
    sFileU = sFileA;
  } else if( sFileT != "" ) {               // 2. use voc file set to use with test mode in settings
    sFileU = sFileT;
  } else if( sFile != "" ) {                // 3. use voc file last used in normal mode
    sFileU = sFile;
  } else {                                  // 4. bail out, no file to use !
    sErrTit = tr( "Error - No File" );
    sErrTxt = tr( "There was no vocabulary file specified. You can\n"
                  "- pass the file name as a command line argument,\n"
                  "- set it in the settings screen or\n"
                  "- the file last used in normal mode is used.\n \n"
                  "None of this worked here.\n"
                  "Have you ever opened the program in normal mode\n"
                  "(without any command line arguments) and created\n"
                  "a vocabulary file at all?\n"
                  "Otherwise there might be a problem with the program's\n"
                  "configuration file. On Linux this usually is:\n"
                  "~/.config/qvtwfc/qvtwfc.conf\n \n"
                  "Aborting..." );
    errorNOut();
  }

  if( iIterA > 1 ) {                        // num words to ask were provided as command line argument and > 1 !
    iIterU = iIterA;
  } else if( iIterT > 1 ) {                 // use num words to ask from settings
    iIterU = iIterT;
  } else {                                  // must never happen
    iIterU = 2;
  }
  sI = "/" + QString::number( iIterU );     // '/'+number iterations as string for counter

  fileChkPrep();
}


void Qvtwfct::errorNOut()                   // show error message and quit
{
  QMessageBox::information( this, sErrTit, sErrTxt  + tr( "\n \nThe program will now exit..." ) );
  quitDo();
}


// ****************************************************************
// **  file open functions/slots                                 **
// ****************************************************************
void Qvtwfct::fileChkPrep()
{
  rwf->readFileChk( sFileU, 1 );            // continue, check file (exists, readable?) n try to open, mode 1 = flash card
}

void Qvtwfct::readAbort( const QString& sD, const QString& sD2, int iD )
{
  sErrTit = sD;
  sErrTxt = sD2;
  errorNOut();
}

void Qvtwfct::readWErr( const QString& sErrTit, const QString& sErrTxt, int iD2, const QString& sD1, const QString & sD2 )  // file read with errors, iD2 = error code
{
  QMessageBox::warning( this, sErrTit, sErrTxt );
  sO = sD1;
  sT = sD2;
  trainPrep();                                // prepare trainings session
}

void Qvtwfct::readOk( const QString& sD1, const QString & sD2 )  // file read, set language
{
  sO = sD1;
  sT = sD2;
  trainPrep();                                // prepare trainings session
}

void Qvtwfct::trainPrep()                     // prepare trainings session
{
  wordpair = rwf->wordpair;
  iWpCnt = wordpair.count();
  ui.l_OrigLang->setText( sO );
  ui.l_TransLang->setText( sT );
  fcLoopBegin();
}


// ****************************************************************
// **  test loop functions/slots                                 **
// ****************************************************************
void Qvtwfct::fcClear()                     // clear widget on start up
{
  ui.l_OrigLang->setText( "" );
  ui.l_OrigVoc->setText( "" );
  ui.l_TransLang->setText( "" );
  ui.tB_TransVoc->setText( "" );
  ui.l_Cnt->setText( "" );
}

void Qvtwfct::fcLoopBegin()                // begin loop (test run)
{
  qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
  iCnt = 0;                                 // counter of word pairs shown
  fcGetPair();
}

void Qvtwfct::fcGetPair()                   // get word pair: original and translation
{
  iLine = qrand() % iWpCnt;
  fcShowOrig();
}

void Qvtwfct::fcShowOrig()                  // show original lang word
{
  ui.l_OrigVoc->setText( wordpair.at( iLine).orig + " , " + wordpair.at( iLine).type );
  ui.tB_TransVoc->setText( "" );
  ui.l_Cnt->setText( QString::number( iCnt + 1 ) + sI );
  iNextStep = 2;
}

void Qvtwfct::fcShowTrans()                 // show translation
{
  ui.tB_TransVoc->setText( wordpair.at( iLine).trans );
  iNextStep = 1;
  iCnt = iCnt + 1;
}

void Qvtwfct::fcLoopNext()                  // next clicked, show translation, next new word or exit
{
  if( iCnt == iIterU ) {
    quitPrep();
  } else if( iNextStep == 2 ) {
    fcShowTrans();
  } else if( iNextStep == 1 ) {
    fcGetPair();
  } else {
    QMessageBox::warning( this, tr( "Ooops !" ), 
                                tr( "How could this happen ?\n"
                                    "You left the test loop !\n"
                                    "This must never happen !\n \n"
                                    "Aborting..." ) );
    quitPrep();
  }
}


#include "qvtwfct.moc"
