/****************************************************************************
 **
 **  Copyright (C) 2008 Helge Kortendieck, alan@kasmithee.de
 **
 **  settings (configuration) dialog
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include "settings.h"

settings::settings( QDialog* parent, Qt::WindowFlags fl )
        : QDialog( parent, fl )
{
  uis.setupUi( &wSetDlg );                  // settings dialog
  connect(uis.pB_Save, SIGNAL( clicked() ), this, SLOT( setDlgSave() ) );
  connect(uis.pB_Cancel, SIGNAL( clicked() ), this, SLOT( setDlgCancel() ) );
  connect(uis.pB_Dict, SIGNAL( clicked() ), this, SLOT( setDlgQueryDictServer() ) );
  connect(uis.cB_Dict, SIGNAL( stateChanged( int ) ), this, SLOT( setDlgDictToggle() ) );
  connect(uis.cB_Dict, SIGNAL( stateChanged( int ) ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.lE_DictIp, SIGNAL( editingFinished() ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.spB_DictPort, SIGNAL( valueChanged( int ) ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.tW_Dicts, SIGNAL( itemChanged( QTableWidgetItem* ) ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.pB_OpenVocFileTM, SIGNAL( clicked() ), this, SLOT( setDlgOpenFile() ) );
  connect(uis.spb_FontSize, SIGNAL( valueChanged( int ) ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.spb_It, SIGNAL( valueChanged( int ) ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.lE_VocFile, SIGNAL( editingFinished() ), this, SLOT( setDlgChkFile() ) );
  connect(uis.cB_RunFCT, SIGNAL( stateChanged( int ) ), this, SLOT( setDlgEnableSave() ) );
  connect(uis.cB_RunFCT, SIGNAL( stateChanged( int ) ), this, SLOT( setDlgRunTog( int ) ) );

  settingsRead();

  sfcs = new fcs();  // flash card session, set/unset to start when starting a desktop session, signal connected after setting checkbox before showing widget !!

  rwfs = new rw();                          // setup read/write files qobject
  connect( rwfs, SIGNAL( sigReadAbort( const QString&, const QString&, int ) ), this, SLOT( readAbort( const QString&, const QString&, int ) ) ); // fatal err while opening file
  connect( rwfs, SIGNAL( sigReadOk( const QString&, const QString& ) ), this, SLOT( readOk( const QString&, const QString& ) ) ); // reading file done, no errors

  setDlgOpen();                             // short timer to allow GUI to come up before proceeding...
}

settings::~settings()
{
}


void settings::settingsRead()
{
  dictsConf.clear();
  QSettings settings("qvtwfc", "qvtwfc");

  settings.beginGroup( "mainwindow" );
    iFontSize =  settings.value( "fontsize", 11 ).toInt();
  settings.endGroup();
  settings.beginGroup( "flash_card" );
    bAutoStart = settings.value( "autostart", "false" ).toBool();
    sFileT = settings.value( "filetest", "" ).toString();
    iIter = settings.value( "iter", "2" ).toInt();
  settings.endGroup();
  settings.beginGroup( "dict" );
    bDictUse = settings.value( "dictenable", "false" ).toBool();
    sDictIp = settings.value( "dictip", "localhost").toString();
    iDictPort = settings.value( "dictport", 2628 ).toInt();
    iD = 0;
    iD = settings.beginReadArray("dicts");
    if( iD > 0 ) {
      for (int i = 0; i < iD; ++i) {
        settings.setArrayIndex(i);
        dictdb.selected = true;
        dictdb.name = settings.value( "dictName" ).toString();
        dictdb.desc = settings.value( "dictDesc" ).toString();
        dictsConf.append( dictdb );
      }
    }
    settings.endArray();
  settings.endGroup();
  settings.beginGroup( "settingsdialog" );
    QPoint pos = settings.value( "pos", QPoint(0, 0) ).toPoint();
    QSize size = settings.value( "size", QSize(520, 530) ).toSize();
  settings.endGroup();

  resize(size);
  move(pos);
}

void settings::settingsWrite()
{
  QSettings settings("qvtwfc", "qvtwfc");

  settings.beginGroup( "mainwindow" );
    settings.setValue( "fontsize", uis.spb_FontSize->value() );
  settings.endGroup();
  settings.beginGroup( "flash_card" );
    settings.setValue( "autostart", uis.cB_RunFCT->isChecked() );
    settings.setValue( "filetest", uis.lE_VocFile->text() );
    settings.setValue( "iter", uis.spb_It->value() );
  settings.endGroup();
  settings.beginGroup( "dict" );
    settings.setValue( "dictenable", uis.cB_Dict->isChecked() );
    settings.setValue( "dictip", uis.lE_DictIp->text() );
    settings.setValue( "dictport", uis.spB_DictPort->value() );
    settings.beginWriteArray("dicts");
    if( bDictUse == true ) {                                      // dictionaries to save
      if( dictsToQuery.count() > 0 ) {                                // redundant...
        if( iD > 0 ) {
          for (int i = 0; i < dictsToQuery.size(); ++i) {
            settings.setArrayIndex(i);
            settings.setValue("dictName", dictsToQuery.at(i).name );  // name of dictionary
            settings.setValue("dictDesc", dictsToQuery.at(i).desc );  // description of dictionary
          }
        }
      }
    }
    settings.endArray();
  settings.endGroup();
  settings.beginGroup( "settingsdialog" );
    settings.value( "pos", pos() );
    settings.value( "size", size() );
  settings.endGroup();

  settings.sync();
}


// ****************************************************************
// **  settings dialog functions/slots                           **
// ****************************************************************
void settings::setDlgOpen()                 // button 'settings' clicked, show settings dialog
{
  QFont font;
    font.setPointSize( iFontSize );
    wSetDlg.setFont( font );
  uis.spb_FontSize->setValue( iFontSize );
  uis.cB_Dict->setChecked( bDictUse );
  if( bDictUse == true ) {
    uis.lE_DictIp->setText( sDictIp );
    uis.spB_DictPort->setValue( iDictPort );
    QTimer::singleShot(200, this, SLOT( setDlgQueryDictServer() ) );  // get available dicts from server
  }
  uis.spb_It->setValue( iIter );
  uis.lE_VocFile->setText( sFileT );
    sFileTOrig = sFileT;
  uis.cB_RunFCT->setChecked( bAutoStart );
  connect(sfcs, SIGNAL( sigState( bool, bool, bool, bool, const QString&, const QString&, int ) ), this, SLOT( setDlgRunState( bool, bool, bool, bool, const QString&, const QString&, int ) ) );
  sfcs->state();                                  // set state of autorun in supported desktops
  if( sFileT != "" ) {
    setDlgChkFile();                              // voc file exists and is readable?
  }

  QTimer::singleShot(300, this, SLOT( setDlgDisableSave() ) );  // disable save button on first open, delayed to allow first dict server populating
  wSetDlg.show();
}

void settings::setDlgQueryDictServer()      // query dict server for available dicts
{
  emit sigSetDlgGetDicts( uis.lE_DictIp->text(), uis.spB_DictPort->value() );
}

void settings::setDlgReceiveAvailDicts( const QList<DictDef>& dictsD, const QList<DictDef>& dictsD2 ) // cancel clicked in settings dialog, reset original figures
{
  dictsOnServer = dictsD;
  dictsToQuery = dictsD2;
  if( dictsOnServer.count() > 0 ) {
    uis.f_Dict->setEnabled( true );
    setDlgDictWEnable();
    setDlgDictPop();
  } else {
    uis.f_Dict->setEnabled( false );
    setDlgDictWDisable();
  }
}

void settings::setDlgCancel()               // cancel clicked in settings dialog, reset original figures
{
  setDlgClose();
}

void settings::setDlgDisableSave()          // disable save button on first open, delayed to allow first dict server populating
{
  uis.pB_Save->setEnabled( false );
}

void settings::setDlgEnableSave()           // enable save button
{
  uis.pB_Save->setEnabled( true );
}

void settings::setDlgSave()                 // save clicked in settings dialog
{
  sFileT = uis.lE_VocFile->text();              // vocabulary file to use if not default (last used in normal mode)
  if( sFileT.isEmpty() == false ) {
    rwfs->readFileChk( sFileT, 1 );             // continue, check file: exists, readable ?        ***************************************** this is wrong !!!
  }
  dictsToQuery.clear();
  if( bDictUse == true ) {                      // see if we have any dictionaries to save
    iD = uis.tW_Dicts->rowCount();
    if( iD > 0 ) {
      for ( i = 0; i < iD; ++i ) {
        if( uis.tW_Dicts->item( i, 0 )->checkState() == Qt::Checked ) {              // found selected dictionary, store
          dictdb.selected = true;
          dictdb.name = uis.tW_Dicts->item( i, 0 )->text();
          dictdb.desc = uis.tW_Dicts->item( i, 1 )->text();
          dictsToQuery.append( dictdb );
        }
      }
      if( dictsToQuery.count() == 0 ) {                             // no dictionary is checked
        bDictUse = false;
      }
    } else {                                                    // set using dict to false, if no dictionary available
      bDictUse = false;
    }
  }

  settingsWrite();
  emit sigSetDlgReReadSettings();                  // let main and dic re-read settings
  setDlgClose();
}

void settings::setDlgDictToggle()           // toggle state of dict frame: enabled/disabled
{
  if( uis.cB_Dict->isChecked() ) {
    uis.f_Dict->setEnabled( true );
  } else {
    uis.f_Dict->setEnabled( false );
    setDlgDictWDisable();
  }
}

void settings::setDlgDictWDisable()         // disable dict widget in settings dialog
{
  uis.tW_Dicts->clear();
  uis.tW_Dicts->setColumnCount( 0 );
  uis.tW_Dicts->setRowCount( 0 );
  uis.l_Dict->setEnabled( false );
  uis.tW_Dicts->setEnabled( false );
  uis.tW_Dicts->setToolTip( tr( "No dictionaries available" ) );
}

void settings::setDlgDictWEnable()          // enable dict widget in settings dialog (incl. clear)
{
  uis.tW_Dicts->clear();
  uis.l_Dict->setEnabled( true );
  uis.tW_Dicts->setEnabled( true );
  uis.tW_Dicts->setRowCount( dictsOnServer.size() );
  uis.tW_Dicts->setColumnCount( 2 );
  uis.tW_Dicts->setToolTip( tr( "Select the dictionary(ies) to be used when looking up words" ) );
}

void settings::setDlgDictErr()              // error while querying the dict server
{
  QMessageBox::warning( this, tr( "Error - with dict server !" ), sDictTT + tr( "\nCheck the dict server settings and retry." ) );
}

void settings::setDlgDictPop()              // populate dict widget with all available dict servers, so that they can be selected
{
  slD.clear();                                // set headers 'Name' and 'Description'
  slD.append( tr( "Name" ) );
  slD.append( tr( "Description" ) );
  uis.tW_Dicts->setHorizontalHeaderLabels( slD );
  for ( i = 0; i < dictsOnServer.count(); i++ ) {
    sD = dictsOnServer.at( i ).name;
    sD2 = dictsOnServer.at( i ).desc;
//  QMessageBox::information( this, "debug - in setDlgDictPop !", sD + " , " + sD2 );       // **************************************************   debug   ******
    QTableWidgetItem *newItem = new QTableWidgetItem( sD );             // dictionary name
    newItem->setFlags( Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );   // Qt::ItemIsUserCheckable
    newItem->setCheckState( Qt::Unchecked );
    uis.tW_Dicts->setItem( i, 0, newItem);
    QTableWidgetItem *newItem2 = new QTableWidgetItem( sD2 );           // dictionary name
    uis.tW_Dicts->setItem( i, 1, newItem2);
  }

//cout << "setDlgDictPop(): dictsUse= " << QString::number( dictsToQuery.count() ).toStdString() << endl;                               // *****  debug  *****
  if( dictsToQuery.count() > 0 ) {            // have selected dictionary(ies) -> see if they are available and set them checked
    qtwiD = new QTableWidgetItem( 0 );
    for ( i = 0; i < dictsToQuery.count(); i++ ) {
      for ( i2 = 0; i2 < uis.tW_Dicts->rowCount(); i2++ ) {
        qtwiD = uis.tW_Dicts->item( i2, 0 );
        if( dictsToQuery.at( i ).name == qtwiD->text() ) {
          qtwiD->setCheckState( Qt::Checked );
        }
      }
    }
  }
  uis.tW_Dicts->resizeRowsToContents();
  uis.tW_Dicts->resizeColumnsToContents();
}

void settings::setDlgOpenFile()             // file dialog to select file to use in test mode run
{
  sD = uis.lE_VocFile->text();
  sDir = QDir::homePath();
  if( sD.length() > 0 ) {
    QFileInfo fi( sD );                           // get start dir for dialog
    if( fi.exists() ) {
      sDir = fi.absoluteFilePath();
    } else {
      sD2 = fi.absolutePath();
      QFileInfo fi2( sD2 );
      if( fi2.exists() == true ) {
        sDir = sD2;
      }
    }
  }
  sFileT = QFileDialog::getOpenFileName(this, tr( "Open vocabulary file to use in test session mode" ), sDir, "" );

  uis.lE_VocFile->setText( sFileT );
  if( sFileT == "" ) {                          // file dialog canceled -> re-set
    sFileT = sD;
    uis.lE_VocFile->setText( sFileT );
  }
  if( sFileTOrig != uis.lE_VocFile->text() ) {            // lE entry now different as before clicking on file dialog button
    setDlgChkFile();                            // continue, check file: exists, readable ?
  }
}

void settings::setDlgChkFile()              // check test mode voc file: exists, readable ?                                      NEED RETURN EVALUATION HERE !!!!
{
  if( sFileTOrig != uis.lE_VocFile->text() ) {    // only do anything, if voc file was changed
    if( uis.lE_VocFile->text().length() > 0 ) {   // something to check
      rwfs->readFileChk( sFileT, 1 );
    } else {
      setDlgEnableSave();
    }
  }
}

void settings::setDlgRunTog( int iRun )     // run at start of session toggled, 0 = unchecked, 2 = checked
{
  if( iRun == 0 ) {                             // unchecked
    sfcs->unset();
  } else if( iRun == 2 ) {                      // checked
    sfcs->set();
  }
}

void settings::setDlgRunState( bool bKde, bool bKdeSet, bool bGnome, bool bGnomeSet, const QString& sErrTit, const QString& sErrTxt, int iErr )
{                                           // update settings widget acc. desktops found
  uis.cB_Kde->setChecked( bKde );
  uis.cB_Kde_Start->setChecked( bKdeSet );
  uis.cB_Gnome->setChecked( bGnome );
  uis.cB_Gnome_Start->setChecked( bGnomeSet );

  if( iErr > 0 ) {
    QMessageBox::warning( this, QString::number( iErr) + ": " + sErrTit, sErrTxt );
  } else if( uis.cB_RunFCT->isChecked() ) {
    if( ( uis.cB_Kde->isChecked() and !uis.cB_Kde_Start->isChecked() ) or ( uis.cB_Gnome->isChecked() and  !uis.cB_Gnome_Start->isChecked() ) ) {
      QMessageBox::warning( this, tr( "Error - inconsistency detected !" ), 
                                  tr( "You would like to run qvtwfc in flash card/test mode "
                                      "when starting a desktop session, but it is not set on "
                                      "at least one of the installed and supported desktops!\n \n"
                                      "Please toggle the checkbox\n"
                                      "'run flash card test when starting a desktop session'\n"
                                      "twice to either solve the problem or, more likely, to get more "
                                      "information for the failure." ) );
    }
  }
}

void settings::setDlgClose()                // close settings dialog
{
  disconnect(sfcs, SIGNAL( sigState( bool, bool, bool, bool, const QString&, const QString&, int ) ), this, SLOT( setDlgRunState( bool, bool, bool, bool, const QString&, const QString&, int ) ) );
  sfcs->deleteLater();
  emit sigSetDlgClose();
  wSetDlg.close();
}


// **************************************************************************
// **  rw handling - checking voc file                                     **
// **************************************************************************
void settings::readAbort( const QString& sErrTit, const QString& sErrTxt, int iErr )  // fatal error while opening file
{
  QMessageBox::warning( this, sErrTit, sErrTxt + 
                              tr( "\n \nPlease select another vocabulary file for the test sessions or "
                                  "leave blank to use the vocabulary file last worked on in 'normal' mode.\n \n"
                                  "(Note: It may indeed be, that a formerly valid vocabulary file is no longer available, "
                                  "even though, you did not change this setting.)\n \n"
                                  "The file selection will be cleared/re-setted...\n"
                                  "However, you can close this settings dialog via the 'Cancel' button to preserve the old entry, "
                                  "but all changes done so far will then be lost and "
                                  "it is not likely, that the test session will function properly!" ) );
  uis.lE_VocFile->setText( "" );
}

void settings::readOk( const QString& sD1, const QString & sD2 ) // finished reading file, no errors
{
  setDlgEnableSave();
}


#include "settings.moc"
